@@ -1,8 +1,8 @@
-/*  $Id: fastrm.c 6155 2003-01-19 19:58:25Z rra $
+/*  $Id: fastrm.c 8576 2009-08-18 13:55:11Z iulius $
 **
 **  Delete a list of filenames or tokens from stdin.
 **
-**  Originally written by <kre@munnari.oz.au> (to only handle files)
+**  Originally written by <kre@munnari.oz.au> (to only handle files).
 **
 **  Files that can't be unlinked because they didn't exist are considered
 **  okay.  Any error condition results in exiting with non-zero exit
@@ -24,8 +24,8 @@
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/qio.h"
-#include "libinn.h"
-#include "storage.h"
+#include "inn/libinn.h"
+#include "inn/storage.h"
 
 /* We reject any path names longer than this. */
 #define MAX_DIR_LEN 2048
@@ -86,8 +86,8 @@ file_compare(const void *a, const void *b)
 {
     const char *f1, *f2;
 
-    f1 = *((const char **) a);
-    f2 = *((const char **) b);
+    f1 = *((const char *const *) a);
+    f2 = *((const char *const *) b);
     return strcmp(f1, f2);
 }
 
@@ -146,6 +146,8 @@ filelist_empty(filelist *list)
 {
     int i;
 
+    if (list->files == NULL)
+        return;
     for (i = 0; i < list->count; i++)
         free(list->files[i]);
     list->count = 0;
@@ -215,8 +217,6 @@ get_line(QIOSTATE *qp)
     while (QIOtoolong(qp) || (p != NULL && strlen(p) >= MAX_DIR_LEN)) {
         warn("line %d too long", count);
         error_count++;
-        while (p == NULL && QIOtoolong(qp))
-            p = QIOread(qp);
         p = QIOread(qp);
     }
     if (p == NULL) {
