@@ -1,4 +1,4 @@
-/*  $Id: clibrary.h 6704 2004-05-16 19:46:20Z rra $
+/*  $Id: clibrary.h 8393 2009-03-26 19:14:45Z iulius $
 **
 **  Here be declarations of routines and variables in the C library.
 **  Including this file is the equivalent of including all of the following
@@ -78,25 +78,48 @@
 
 BEGIN_DECLS
 
-/* Provide prototypes for functions not declared in system headers.  Use the
-   NEED_DECLARATION macros for those functions that may be prototyped but
-   implemented incorrectly. */
+/* Handle defining fseeko and ftello.  If HAVE_FSEEKO is defined, the system
+   header files take care of this for us.  Otherwise, see if we're building
+   with large file support.  If we are, we have to provide some real fseeko
+   and ftello implementation; declare them if they're not already declared and
+   we'll use replacement versions in libinn.  If we're not using large files,
+   we can safely just use fseek and ftell.
+
+   We'd rather use fseeko and ftello unconditionally, even when we're not
+   building with large file support, since they're a better interface.
+   Unfortunately, they're available but not declared on some systems unless
+   building with large file support, the AC_FUNC_FSEEKO Autoconf function
+   always turns on large file support, and our fake declarations won't work on
+   some systems (like HP_UX).  This is the best compromise we've been able to
+   come up with. */
 #if !HAVE_FSEEKO
+# if DO_LARGEFILES
+#  if !HAVE_DECL_FSEEKO
 extern int              fseeko(FILE *, off_t, int);
-#endif
-#if !HAVE_FTELLO
+#  endif
+#  if !HAVE_DECL_FTELLO
 extern off_t            ftello(FILE *);
+#  endif
+# else
+#  define fseeko(f, o, w) fseek((f), (long)(o), (w))
+#  define ftello(f)       ftell(f)
+# endif
 #endif
-#if !HAVE_HSTRERROR
-extern const char *     hstrerror(int);
+
+/* Provide prototypes for functions not declared in system headers.  Use the
+   HAVE_DECL macros for those functions that may be prototyped but
+   implemented incorrectly or implemented without a prototype. */
+#if !HAVE_ASPRINTF
+extern int              asprintf(char **, const char *, ...);
+extern int              vasprintf(char **, const char *, va_list);
 #endif
 #if !HAVE_MKSTEMP
 extern int              mkstemp(char *);
 #endif
-#if !HAVE_PREAD
+#if !HAVE_DECL_PREAD
 extern ssize_t          pread(int, void *, size_t, off_t);
 #endif
-#if !HAVE_PWRITE
+#if !HAVE_DECL_PWRITE
 extern ssize_t          pwrite(int, const void *, size_t, off_t);
 #endif
 #if !HAVE_SETENV
@@ -105,20 +128,20 @@ extern int              setenv(const char *, const char *, int);
 #if !HAVE_SETEUID
 extern int              seteuid(uid_t);
 #endif
-#if NEED_DECLARATION_SNPRINTF
+#if !HAVE_DECL_SNPRINTF
 extern int              snprintf(char *, size_t, const char *, ...)
     __attribute__((__format__(printf, 3, 4)));
 #endif
-#if !HAVE_STRERROR
-extern const char *     strerror(int);
-#endif
-#if !HAVE_STRLCAT
+#if !HAVE_STRLCAT || !HAVE_DECL_STRLCAT
 extern size_t           strlcat(char *, const char *, size_t);
 #endif
-#if !HAVE_STRLCPY
+#if !HAVE_STRLCPY || !HAVE_DECL_STRLCPY
 extern size_t           strlcpy(char *, const char *, size_t);
 #endif
-#if NEED_DECLARATION_VSNPRINTF
+#if !HAVE_SYMLINK
+extern int              symlink(const char *, const char *);
+#endif
+#if !HAVE_DECL_VSNPRINTF
 extern int              vsnprintf(char *, size_t, const char *, va_list);
 #endif
 
@@ -182,5 +205,14 @@ END_DECLS
 #define ARRAY_SIZE(array)       (sizeof(array) / sizeof((array)[0]))
 #define ARRAY_END(array)        (&(array)[ARRAY_SIZE(array)])
 
+/* C99 requires va_copy.  Older versions of GCC provide __va_copy.  Per the
+   Autoconf manual, memcpy is a generally portable fallback. */
+#ifndef va_copy
+# ifdef __va_copy
+#  define va_copy(d, s)         __va_copy((d), (s))
+# else
+#  define va_copy(d, s)         memcpy(&(d), &(s), sizeof(va_list))
+# endif
+#endif
 
 #endif /* !CLIBRARY_H */
