@@ -1,4 +1,4 @@
-/*  $Id: setproctitle.h,v 1.1 2002/08/29 05:17:55 rra Exp $
+/*  $Id: setproctitle.h 8302 2009-01-29 19:46:28Z iulius $
 **
 **  Set things up for setproctitle portably.
 **
@@ -12,7 +12,7 @@
 
 #include "config.h"
 
-#if !HAVE_SETPROCTITLE
+#if !HAVE_SETPROCTITLE || !HAVE_DECL_SETPROCTITLE
 void setproctitle(const char *format, ...);
 #endif
 
