@@ -1,4 +1,4 @@
-/*  $Id: status.c 7547 2006-08-26 06:18:14Z eagle $
+/*  $Id: status.c 8178 2008-11-20 15:23:49Z iulius $
 **
 **  Periodic status reporting.
 */
@@ -6,7 +6,9 @@
 #include "clibrary.h"
 #include "portable/socket.h"
 
+#include "inn/network.h"
 #include "inn/innconf.h"
+#include "inn/version.h"
 #include "innd.h"
 #include "innperl.h"
 
@@ -20,9 +22,12 @@
 
 typedef struct _STATUS {
     char		name[SMBUF];
-    char		ip_addr[64];
+    /* ip_addr may contain a concatenation of several
+     * IPv4 and IPv6 addresses. */
+    char		ip_addr[SMBUF];
     bool		can_stream;
     unsigned short	activeCxn;
+    unsigned short      maxCxn;
     unsigned short	sleepingCxns;
     time_t		seconds;
     unsigned long	accepted;
@@ -36,6 +41,7 @@ typedef struct _STATUS {
     unsigned long	Unwanted_f;
     float		Size;
     float		DuplicateSize;
+    float		RejectSize;
     unsigned long	Check;
     unsigned long	Check_send;
     unsigned long	Check_deferred;
@@ -110,9 +116,12 @@ STATUSsummary(void)
   unsigned long		rejected = 0;
   float			size = 0;
   float			DuplicateSize = 0;
+  float			RejectSize = 0;
   int			peers = 0;
   char                  TempString[SMBUF];
   char			*path;
+  char                  other_ip_addr[INET6_ADDRSTRLEN];
+  char                  *p;
   STATUS		*head, *status, *tmp;
   char			str[9];
   time_t		now;
@@ -123,7 +132,7 @@ STATUSsummary(void)
   path = concatpath(innconf->pathlog, STATUS_FILE);
 #endif
   if ((F = Fopen(path, "w", TEMPORARYOPEN)) == NULL) {
-    syslog(L_ERROR, "%s cannot open %s: %m", LogName, path);
+    syswarn("SERVER cant open %s", path);
     return;
   }
 
@@ -136,7 +145,7 @@ STATUSsummary(void)
   fprintf (F, "</HEAD>\n<BODY>\n<PRE>\n") ;
 #endif /* defined(HTML_STATUS) */
 
-  fprintf (F, "%s\n", inn_version_string);
+  fprintf (F, "%s\n", INN_VERSION_STRING);
   fprintf (F, "pid %d started %s\n", (int) getpid(), start_time);
 
   tmp = head = NULL;
@@ -151,13 +160,19 @@ STATUSsummary(void)
     }
     if (status == NULL) {
       status = xmalloc(sizeof(STATUS));
-      peers++;                                              /* a new peer */
+      peers++;                                              /* A new peer. */
       strlcpy(status->name, TempString, sizeof(status->name));
-      strlcpy(status->ip_addr,
-              sprint_sockaddr((struct sockaddr *)&cp->Address),
-              sizeof(status->ip_addr));
+      if (cp->Address.ss_family == 0) {
+          /* Connections from lc.c do not have an IP address. */
+          memset(&status->ip_addr, 0, sizeof(status->ip_addr));
+      } else {
+          network_sockaddr_sprint(status->ip_addr, sizeof(status->ip_addr),
+                                  (struct sockaddr *) &cp->Address);
+      }
       status->can_stream = cp->Streaming;
+      status->maxCxn = cp->MaxCnx;
       status->seconds = status->Size = status->DuplicateSize = 0;
+      status->RejectSize = 0;
       status->Ihave = status->Ihave_Duplicate =
 	status->Ihave_Deferred = status->Ihave_SendIt =
 	status->Ihave_Cybercan = 0;
@@ -177,11 +192,25 @@ STATUSsummary(void)
       else
 	tmp->next = status;
       tmp = status;
+    } else if (cp->Address.ss_family != 0) {
+      network_sockaddr_sprint(other_ip_addr, sizeof(other_ip_addr),
+                              (struct sockaddr *) &cp->Address);
+      /* Concatenate the addresses if it is a new one.
+       * Make sure we have enough space to write the entire new address. */
+      if ( (((p = strstr(status->ip_addr, other_ip_addr)) == NULL)
+            || ((p != NULL) && (p[strlen(other_ip_addr)] != ' ')
+                            && (p[strlen(other_ip_addr)] != '\0')))
+           && (strlen(status->ip_addr) + strlen(other_ip_addr) + 1 <
+               sizeof(status->ip_addr)) ) {
+        strlcat(status->ip_addr, " ", sizeof(status->ip_addr));
+        strlcat(status->ip_addr, other_ip_addr, sizeof(status->ip_addr));
+      }
     }
-    if (Now.time - cp->Started > status->seconds)
-      status->seconds = Now.time - cp->Started;
-    if (Now.time - cp->Started > seconds)
-      seconds = Now.time - cp->Started;
+
+    if (Now.tv_sec - cp->Started > status->seconds)
+      status->seconds = Now.tv_sec - cp->Started;
+    if (Now.tv_sec - cp->Started > seconds)
+      seconds = Now.tv_sec - cp->Started;
     status->accepted += cp->Received;
     accepted += cp->Received;
     status->refused += cp->Refused;
@@ -210,8 +239,10 @@ STATUSsummary(void)
     status->Takethis_Err += cp->Takethis_Err;
     status->Size += cp->Size;
     status->DuplicateSize += cp->DuplicateSize;
+    status->RejectSize += cp->RejectSize;
     size += cp->Size;
     DuplicateSize += cp->DuplicateSize;
+    RejectSize += cp->RejectSize;
     if (CHANsleeping(cp)) {
       sleepingCxns++;
       status->sleepingCxns++;
@@ -235,7 +266,7 @@ STATUSsummary(void)
     fprintf (F, " (%s)", ModeReason);
   
   /* Global configuration */
-  fprintf (F, "\n\nConfiguration file: %s\n\n", _PATH_CONFIG);
+  fprintf (F, "\n\nConfiguration file: %s\n\n", INN_PATH_CONFIG);
 
   fprintf (F, "Global configuration parameters:\n");
   fprintf (F, "              Largest Article: %ld bytes\n", innconf->maxartsize);
@@ -257,14 +288,10 @@ STATUSsummary(void)
   } else {
 	fprintf (F, "               Remember Trash: No\n");
   }
-#if defined(DO_TCL)
-  fprintf (F, "                Tcl filtering: %s\n", 
-	   TCLFilterActive ? "enabled" : "disabled");
-#endif /* defined(DO_TCL) */
-#if defined(DO_PERL)
+#ifdef DO_PERL
   fprintf (F, "               Perl filtering: %s\n", 
 	   PerlFilterActive ? "enabled" : "disabled");
-#endif /* defined(DO_PERL) */
+#endif
 
   fputc ('\n', F) ;
 
@@ -283,17 +310,29 @@ STATUSsummary(void)
 	   rejected, (float) rejected / offered * 100);
   fprintf (F, "      duplicated: %-9ld     %%duplicated: %.1f%%\n",
 	   duplicate, (float) duplicate / offered * 100);
-  fprintf (F, "           bytes: %-7s\n", PrettySize (size + DuplicateSize, str));
+  fprintf (F, "           bytes: %-7s\n", PrettySize (size + DuplicateSize + RejectSize, str));
   fprintf (F, " duplicated size: %-7s  %%duplicated size: %.1f%%\n",
 	   PrettySize(DuplicateSize, str), (float) DuplicateSize / size * 100);
+  fprintf (F, "   rejected size: %-7s    %%rejected size: %.1f%%\n",
+	   PrettySize(RejectSize, str), (float) RejectSize / size * 100);
   fputc ('\n', F) ;
-  
+
+  if(innconf->logstatus) {
+    notice ("%s status seconds %ld accepted %ld "
+            "refused %ld rejected %ld duplicate %ld "
+            "accepted size %.0f duplicate size %.0f rejected size %.0f\n",
+                "ME", (long) seconds, accepted,
+                refused, rejected, duplicate,
+                size, DuplicateSize, RejectSize);
+  }
+
   /* Incoming Feeds */
   for (status = head ; status != NULL ;) {
     fprintf (F, "%s\n",                      status->name);
+    fprintf (F, " ip address: %s\n",         status->ip_addr);
     fprintf (F, "    seconds: %-7ld  ",      (long) status->seconds);
-    fprintf (F, "      duplicates: %-7ld ",  status->Duplicate);
-    fprintf (F, "    ip address: %s\n",      status->ip_addr);
+    fprintf (F, "      duplicates: %-5ld ",  status->Duplicate);
+    fprintf (F, "max allowed cxns: %d\n",    status->maxCxn);
     fprintf (F, "    offered: %-7ld  ",
 	     status->accepted + status->refused + status->rejected);
     fprintf (F, "   uw newsgroups: %-7ld ",  status->Unwanted_g);
@@ -312,24 +351,35 @@ STATUSsummary(void)
     fprintf (F, "       size: %-8s ",        PrettySize(status->Size, str));
     fprintf (F, "       bad sites: %-7ld ", status->Unwanted_s);
     fprintf (F, "duplicate size: %s\n", PrettySize(status->DuplicateSize, str));
+    fprintf (F, "reject size: %-8s\n",       PrettySize(status->RejectSize, str));
     fprintf (F, "  Protocol:\n");
     fprintf (F, "      Ihave: %-6ld SendIt[%d]: %-6ld    Got[%d]: %-6ld Deferred[%d]: %ld\n",
-	     status->Ihave, NNTP_SENDIT_VAL, status->Ihave_SendIt,
-	     NNTP_HAVEIT_VAL, status->Ihave_Duplicate, NNTP_RESENDIT_VAL,
+	     status->Ihave, NNTP_CONT_IHAVE, status->Ihave_SendIt,
+	     NNTP_FAIL_IHAVE_REFUSE, status->Ihave_Duplicate, NNTP_FAIL_IHAVE_DEFER,
 	     status->Ihave_Deferred);
     fprintf (F, "      Check: %-6ld SendIt[%d]: %-6ld    Got[%d]: %-6ld Deferred[%d]: %ld\n",
-	     status->Check, NNTP_OK_SENDID_VAL, status->Check_send,
-	     NNTP_ERR_GOTID_VAL, status->Check_got, NNTP_RESENDID_VAL,
+	     status->Check, NNTP_OK_CHECK, status->Check_send,
+	     NNTP_FAIL_CHECK_REFUSE, status->Check_got, NNTP_FAIL_CHECK_DEFER,
 	     status->Check_deferred);
     fprintf (F, "   Takethis: %-6ld     Ok[%d]: %-6ld  Error[%d]: %-6ld\n",
-	     status->Takethis, NNTP_OK_RECID_VAL, status->Takethis_Ok,
-	     NNTP_ERR_FAILID_VAL, status->Takethis_Err);
+	     status->Takethis, NNTP_OK_TAKETHIS, status->Takethis_Ok,
+	     NNTP_FAIL_TAKETHIS_REJECT, status->Takethis_Err);
     if (innconf->refusecybercancels) {
         fprintf (F, "   Cancelrejects:    Ihave[%d]: %-6ld  Check[%d]: %-6ld\n",
-	     NNTP_HAVEIT_VAL, status->Ihave_Cybercan,
-	     NNTP_ERR_GOTID_VAL, status->Check_cybercan);
+	     NNTP_FAIL_IHAVE_REFUSE, status->Ihave_Cybercan,
+	     NNTP_FAIL_CHECK_REFUSE, status->Check_cybercan);
     }
     fputc ('\n', F) ;
+
+    if(innconf->logstatus) {
+      notice ("%s status seconds %ld accepted %ld "
+              "refused %ld rejected %ld duplicate %ld "
+              "accepted size %.0f duplicate size %.0f rejected size %.0f\n",
+                  status->name, (long) status->seconds, status->accepted,
+                  status->refused, status->rejected, status->Duplicate,
+                  status->Size, status->DuplicateSize, status->RejectSize);
+    }
+
     tmp = status->next;
     free(status);
     status = tmp;
@@ -341,6 +391,7 @@ STATUSsummary(void)
 #endif /* defined(HTML_STATUS) */
 
   Fclose(F);
+  free(path);
 }
 
 void
