@@ -1,104 +1,122 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Wed Dec 27 17:10:18 1995
- * Project:     INN (innfeed)
- * File:        misc.c
- * RCSId:       $Id: misc.c,v 1.7.2.1 2000/06/29 22:25:45 rra Exp $
- *
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Description: 
- * 
- */
-
-#if ! defined (lint)
-static const char *rcsid = "$Id: misc.c,v 1.7.2.1 2000/06/29 22:25:45 rra Exp $" ;
-static void use_rcsid (const char *rid) {   /* Never called */
-  use_rcsid (rcsid) ; use_rcsid (rid) ;
-}
-#endif
+/*  $Id: misc.c 8243 2008-12-21 11:50:47Z iulius $
+**
+**  Helper routines for the innfeed program.
+**
+**  Written by James Brister <brister@vix.com>
+*/
 
+#include "innfeed.h"
 #include "config.h"
+#include "clibrary.h"
 
-
-#include <stdlib.h>
-#include <sys/types.h>
-#include <stdarg.h>
-#include <stdio.h>
-#include <netdb.h>
-#include <ctype.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <assert.h>
-#include <netinet/in.h>
-#include <arpa/nameser.h>
-#include <resolv.h>
-#include <fcntl.h>
+#include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <netdb.h>
+#include <netinet/in.h>
+#include <signal.h>
 #include <syslog.h>
 #include <sys/param.h>
-#include <limits.h>
-
-#if defined (HAVE_UNISTD_H)
-#include <unistd.h>
-#endif
-
-#if defined (_AIX41)
+#include <sys/stat.h>
 #include <time.h>
+
+/* FIXME: Default to a max length of 256 characters for path names if the
+   host headers doesn't give better information.  Should be replaced by the
+   code from Stevens. */
+#ifndef PATH_MAX
+# define PATH_MAX 256
 #endif
 
-#include <signal.h>
+#include "inn/messages.h"
+#include "inn/libinn.h"
 
-#include "configdata.h"
-#include "clibrary.h"
-#include "libinn.h"
-#include "misc.h"
-#include "msgs.h"
 #include "endpoint.h"
+#include "misc.h"
 #include "tape.h"
 
-u_int openfds ;
-char *program ;                 /* this should be set to argv[0] */
+unsigned int openfds ;
 int debuggingOutput ;
-u_int loggingLevel ;
+unsigned int loggingLevel ;
 char **PointersFreedOnExit ;
-static void log (int level, const char *fmt, va_list args) ;
+
+char *timeToStringFormat = 0;
+bool debuggingDump = true ;
+extern void (*gPrintInfo) (void) ;
+void (*gCleanUp) (void) = 0 ;
 
 
-
-
-
-int maxFds (void)
+/* Log a message to stderr, called from warn or die.  Mostly the same as the
+   standard message_log_stderr, but prepends the date to each line. */
+void
+error_log_stderr_date(int len UNUSED, const char *fmt, va_list args, int err)
 {
-  static int size = 0 ;
-  size = getfdcount () ;
-  return size;
+    char timebuff[30];
+    time_t now;
+    struct tm *tm;
+
+    now = time(NULL);
+    tm = localtime(&now);
+    strftime(timebuff, sizeof(timebuff), "%Y-%m-%d %H:%M:%S", tm);
+    fprintf(stderr, "%s %s: ", timebuff,
+            (message_program_name ? message_program_name : "UNKNOWN"));
+    vfprintf(stderr, fmt, args);
+    if (err) fprintf(stderr, ": %s", strerror(err));
+    fprintf(stderr, "\n");
+}
+
+/* If desired, print out the state of innfeed, call a cleanup function, and
+   then dump core.  Used as an exit handler for die. */
+int
+dump_core(void)
+{
+#if SNAPSHOT_ON_DIE
+    if (debuggingDump && gPrintInfo != NULL)
+        (*gPrintInfo)();
+#endif
+
+    if (gCleanUp != NULL)
+        (*gCleanUp)();
+  
+    chdir(getTapeDirectory());
+  
+    sleep(5);
+    abort();
+
+    /* Not reached. */
+    return 1;
+}
+
+/* An alternate version of die, used when we don't want to dump core.  This
+   should somehow eventually be phased out to simplify things; it's
+   basically a copy of die() from lib/error.c that ignores the cleanup
+   handler and has innfeed's handlers hard-coded (ugh). */
+void
+logAndExit(int status, const char *format, ...)
+{
+    va_list args;
+    int length;
+
+    va_start(args, format);
+    length = vsnprintf(NULL, 0, format, args);
+    va_end(args);
+    va_start(args, format);
+    error_log_stderr_date(length, format, args, 0);
+    va_end(args);
+    va_start(args, format);
+    message_log_syslog_err(length, format, args, 0);
+    va_end(args);
+    exit(status);
 }
 
 
-void d_printf (u_int level, const char *fmt, ...) 
+
+void d_printf (unsigned int level, const char *fmt, ...) 
 {
   static pid_t myPid ;
   char timeString [30] ;
   time_t now ;
+  struct tm *tm ;
   va_list ap ;
     
   if (myPid == 0)
@@ -108,40 +126,17 @@ void d_printf (u_int level, const char *fmt, ...)
     return ;
   
   now = theTime() ;
-  strcpy (timeString, ctime (&now) + 4) ; /* strip off leading day name */
-  timeString [15] = '\0' ;      /* strip off trailing year and newline */
+  tm = localtime (&now);
+  strftime (timeString, sizeof(timeString), "%b %d %H:%M:%S", tm);
 
   va_start (ap, fmt) ;
   fprintf (stderr, "%s %s[%ld]: ",timeString,
-           (program ? program : "UNKNOWN PROGRAM NAME"), (long) myPid) ;
+           (message_program_name ? message_program_name : "UNKNOWN"),
+           (long) myPid) ;
   vfprintf (stderr, fmt, ap) ;
   va_end (ap) ;
 }
 
-bool debuggingDump = true ;
-extern void (*gPrintInfo) (void) ;
-void (*gCleanUp) (void) = 0 ;
-
-static void log (int level, const char *fmt, va_list args)
-{
-  time_t now = time (NULL) ;
-  char timeString [30] ;
-  char *p = NULL ;
-  int out ;
-  
-  strcpy (timeString,ctime (&now)) ;
-  timeString [24] = '\0' ;
-  
-  fprintf (stderr, "%s %s: ",
-           timeString, (program ? program : "UNKNOWN PROGRAM NAME")) ;
-  out = vfprintf (stderr, fmt, args) ;
-  fprintf (stderr,"\n") ;
-
-  p = malloc (out + 10) ;
-  vsprintf (p,fmt,args) ;
-  syslog (level,"%s",p) ;
-}
-
 void logOrPrint (int level, FILE *fp, const char *fmt, ...)
 {
   va_list ap ;
@@ -156,92 +151,12 @@ void logOrPrint (int level, FILE *fp, const char *fmt, ...)
     {
       char buffer [512] ;      /* gag me */
 
-      vsprintf (buffer,fmt,ap) ;
-      syslog (level,buffer) ;
+      vsnprintf (buffer,sizeof (buffer),fmt,ap) ;
+      syslog (level,"%s",buffer) ;
     }
   va_end (ap) ;
 }
 
-void die (const char *fmt, ...)
-{
-  va_list ap ;
-
-  va_start (ap, fmt) ;
-  log (LOG_ERR,fmt,ap) ;
-  va_end (ap) ;
-
-#if SNAPSHOT_ON_DIE
-  if (debuggingDump && gPrintInfo != NULL)
-    gPrintInfo () ;
-#endif
-
-  if (gCleanUp != NULL)
-    gCleanUp () ;
-  
-  if (CORE_DIRECTORY != NULL)
-    (void) chdir (CORE_DIRECTORY) ;
-  else
-    (void) chdir (getTapeDirectory()) ;
-  
-  sleep (5) ;
-  abort () ;
-}
-
-void warn (const char *fmt, ...)
-{
-  va_list ap ;
-
-  va_start (ap, fmt) ;
-  log (LOG_WARNING,fmt,ap) ;
-  va_end (ap) ;
-}
-
-
-void logAndExit (int exitVal, const char *fmt, ...)
-{
-  va_list ap ;
-
-  va_start (ap,fmt) ;
-  log (LOG_CRIT,fmt,ap) ;
-  va_end (ap) ;
-
-  exit (exitVal) ;
-}
-
-
-
-static const char * const pvt_h_errlist[] = {
-  "Resolver Error 0 (no error)",
-  "Unknown host",                         /* 1 HOST_NOT_FOUND */
-  "Host name lookup failure",             /* 2 TRY_AGAIN */
-  "Unknown server error",                 /* 3 NO_RECOVERY */
-  "No address associated with name",      /* 4 NO_ADDRESS */
-};
-
-static int pvt_h_nerr = (sizeof pvt_h_errlist / sizeof pvt_h_errlist[0]);
-
-#if defined(hpux) || defined(__hpux)
-extern int h_errno;
-#endif
-
-/* return a friendly string for the current value of h_errno. Pinched from
-   Stevens */
-const char *host_err_str (void)
-{
-  static char msgstr [200] ;
-
-  if (h_errno != 0)
-    {
-      if (h_errno > 0 && h_errno < pvt_h_nerr)
-        sprintf (msgstr,"(%s)", pvt_h_errlist[h_errno]) ;
-      else
-        sprintf (msgstr,"(herrno = %d)", h_errno) ;
-    }
-  else
-    msgstr [0] = '\0' ;
-
-  return msgstr ;
-}
 
 
 /* return true if the file exists and is a regular file. */
@@ -281,10 +196,10 @@ bool getNntpResponse (char *p, int *code, char **rest)
   if (p == NULL)
     return false ;
   
-  while (*p && isspace (*p))
+  while (*p && CTYPE (isspace, *p))
     p++ ;
 
-  while (*p && isdigit (*p))
+  while (*p && CTYPE (isdigit, *p))
     {
       digits++ ;
       cd = (cd * 10) + (*p - '0') ;
@@ -297,7 +212,7 @@ bool getNntpResponse (char *p, int *code, char **rest)
   if (*p == '-')
     p++ ;
   
-  while (*p && isspace (*p))
+  while (*p && CTYPE (isspace, *p))
     p++ ;
       
   if (rest)
@@ -309,46 +224,6 @@ bool getNntpResponse (char *p, int *code, char **rest)
 }
 
 
-#if 0
-/* pinched from INN main source */
-int
-writev(fd, vp, vpcount)
-    int                 fd;
-    struct iovec        *vp;
-    int                 vpcount;
-{
-    int                 count;
-
-    for (count = 0; --vpcount >= 0; count += vp->iov_len, vp++)
-        if (xwrite(fd, vp->iov_base, vp->iov_len) < 0)
-            return -1;
-    return count;
-}
-
-#endif
-
-
-
-#if defined (DO_NEED_STRDUP)
-
-char *strdup (const char *string)
-{
-  char *p = NULL ;
-  
-  if (string != NULL)
-    {
-      p = MALLOC (strlen (string) + 1) ;
-      assert (p != NULL) ;
-
-      strcpy (p,string) ;
-    }
-
-
-  return p ;
-}
-
-#endif
-
 
 /* Pull out a message id from a response on to a streaming command */
 char *getMsgId (const char *p)
@@ -356,23 +231,19 @@ char *getMsgId (const char *p)
   const char *q ;
   char *rval ;
   
-  while (*p && isspace (*p)) p++ ;
-  while (*p && !isspace (*p)) p++ ; /* skip response code */
-  while (*p && isspace (*p)) p++ ;
+  while (*p && CTYPE (isspace, *p)) p++ ;
+  while (*p && !CTYPE (isspace, *p)) p++ ; /* skip response code */
+  while (*p && CTYPE (isspace, *p)) p++ ;
 
   if ( *p == '\0' )
     return NULL ;
 
   q = p ;
-  while ( *q && !isspace (*q) )
+  while ( *q && !CTYPE (isspace, *q) )
     q++ ;
-  
-  rval = MALLOC ((size_t) (q - p + 1)) ;
-  assert (rval != NULL) ;
 
-  strncpy (rval,p,(size_t) (q - p)) ;
-  rval [q - p] = '\0' ;
-  
+  rval = xstrndup (p, q - p) ;
+
   return rval ;
 }
 
@@ -383,12 +254,12 @@ char *findNonBlankString (char *ptr, char **tail)
 {
   char *p, *q ;
 
-  for (p = ptr ; *p && isspace (*p) ; p++)
+  for (p = ptr ; *p && CTYPE (isspace, *p) ; p++)
     /* nada */ ;
   if ( ! *p )
     return NULL ;
 
-  for (q = p ; *q && !isspace (*q) ; q++)
+  for (q = p ; *q && !CTYPE (isspace, *q) ; q++)
     /* nada */ ;
 
   *tail = q ;
@@ -448,7 +319,7 @@ char *mystrtok (char *line, const char *sep)
 void trim_ws (char *string)
 {
   char *p ;
-  u_int len ;
+  unsigned int len ;
 
   assert (string != NULL) ;
 
@@ -456,7 +327,7 @@ void trim_ws (char *string)
   if (len == 0)
     return ;
   
-  for (p = string + len - 1 ; p >= string && isspace (*p) ; p--)
+  for (p = string + len - 1 ; p >= string && CTYPE (isspace, *p) ; p--)
     /* nada */ ;
   *++p = '\0' ;
 }
@@ -513,20 +384,21 @@ void deadBeef (void *base, size_t byteCount)
 bool lockFile (const char *fileName)
 {
   char buff [20] ;
-  char tmpName [MAXPATHLEN], realName [MAXPATHLEN] ;
+  char tmpName [PATH_MAX], realName [PATH_MAX] ;
   char *p ;
   int fd, i ;
   pid_t pid = getpid () ;
 
-  strcpy (realName,fileName) ;
+  strlcpy (realName,fileName,sizeof (realName)) ;
   if ((p = strrchr (realName, '/')) != NULL)
     {
       *p = '\0' ;
-      sprintf (tmpName, "%s/lockf%ld", realName, (long) pid) ;
+      snprintf (tmpName, sizeof(tmpName), "%s/lockf%ld", realName,
+                (long) pid) ;
       *p = '/' ;
     }
   else
-    sprintf (tmpName, "lockf%ld", (long) pid) ;
+    snprintf (tmpName, sizeof(tmpName), "lockf%ld", (long) pid) ;
   
   /* Create the temporary name for the lock file. */
   while ((fd = open (tmpName, O_RDWR | O_CREAT | O_EXCL, 0644)) < 0)
@@ -535,13 +407,13 @@ bool lockFile (const char *fileName)
         {
           default:
             unlink (tmpName) ;
-            syslog (LOG_ERR,NO_OPEN_LOCK,tmpName) ;
+            syswarn ("ME lock file open: %s", tmpName) ;
             return false ;
 
           case EEXIST:
             if (unlink (tmpName) < 0)
               {
-                syslog (LOG_ERR,NO_UNLINK_LOCK,tmpName) ;
+                syswarn ("ME lock file unlink: %s", tmpName) ;
                 return false ;
               }
             break;
@@ -549,10 +421,10 @@ bool lockFile (const char *fileName)
     }
 
   /* stick our pid in the temp file. */
-  sprintf (buff,"%ld\n",(long) pid) ;
+  snprintf (buff,sizeof(buff),"%ld\n",(long) pid) ;
   if (write (fd,buff,(size_t) strlen (buff)) != (int) strlen (buff))
     {
-      syslog (LOG_ERR,NO_WRITE_LOCK_PID) ;
+      syswarn ("ME lock file pid-write") ;
       close (fd) ;
       unlink (tmpName) ;
       return false ;
@@ -565,7 +437,7 @@ bool lockFile (const char *fileName)
       switch (errno) 
         {
           default:              /* opps. bailing out. */
-            syslog (LOG_ERR,NO_LINK_LOCK,realName) ;
+            syswarn ("ME lock file link: %s", realName) ;
             unlink (tmpName) ;
             return false ;
 
@@ -574,7 +446,7 @@ bool lockFile (const char *fileName)
                see if that process is still alive. */
             if ((fd = open (realName,O_RDONLY)) < 0)
               {
-                syslog (LOG_ERR,NO_OPEN_LOCK,realName) ;
+                syswarn ("ME lock file open: %s", realName) ;
                 unlink (tmpName) ;
                 return false ;
               }
@@ -591,7 +463,7 @@ bool lockFile (const char *fileName)
             pid = (pid_t) atol (buff) ;
             if (pid <= 0)
               {
-                syslog (LOG_ERR,BAD_PID,realName,buff) ;
+                warn ("ME lock bad-pid info in %s: %s", realName, buff) ;
                 unlink (tmpName) ;
                 return false ;
               }
@@ -600,7 +472,8 @@ bool lockFile (const char *fileName)
                it's still alive. */
             if (kill (pid,0) == 0)
               {
-                syslog (LOG_ERR,LOCK_EXISTS,realName,(int) pid) ;
+                warn ("ME lock in-use already: %s by pid %ld", realName,
+                      (unsigned long) pid);
                 unlink (tmpName) ;
                 return false ;    /* process is still alive */
               }
@@ -608,7 +481,7 @@ bool lockFile (const char *fileName)
             /* process that took out the lock is gone */
             if (unlink (realName) < 0)
               {
-                syslog (LOG_ERR,NO_UNLINK_LOCK,realName) ;
+                syswarn ("ME lock file unlink: %s", realName) ;
                 unlink (tmpName) ;
                 return false ;
               }
@@ -623,7 +496,7 @@ bool lockFile (const char *fileName)
 
 void unlockFile (const char *lockfile)
 {
-  (void) unlink (lockfile) ;
+  unlink (lockfile) ;
 }
 
 
@@ -697,7 +570,7 @@ bool isOlder (const char *file1, const char *file2)
 
 void freeCharP (char *charp)
 {
-  FREE (charp) ;
+  free (charp) ;
 }
 
 
@@ -719,6 +592,15 @@ const char *boolToString (bool val)
   return val ? "true" : "false" ;
 }
 
+char* timeToString(time_t t, char* buffer, size_t size)
+{
+  static const char defaultFormat[] = "%a %b %d %H:%M:%S %Y" ;
+  const struct tm *const tm = localtime(&t);
+  strftime (buffer, size,
+    timeToStringFormat == 0 ? defaultFormat : timeToStringFormat, tm);
+  return buffer;
+}
+
 void addPointerFreedOnExit (char *pointerToFree)
 {
   static int totalPointers = 0 ;
@@ -726,16 +608,14 @@ void addPointerFreedOnExit (char *pointerToFree)
 
   if (nextPointer == 0 || nextPointer == totalPointers - 1)
     {
-      register int i;
+      int i;
 
       totalPointers += 16 ;
       if (PointersFreedOnExit == NULL)
-	PointersFreedOnExit = ALLOC (char *, totalPointers) ;
+	PointersFreedOnExit = xmalloc (sizeof(char *) * totalPointers) ;
       else
 	PointersFreedOnExit =
-	  REALLOC (PointersFreedOnExit, char *, totalPointers) ;
-
-      ASSERT (PointersFreedOnExit != NULL) ;
+	  xrealloc (PointersFreedOnExit, sizeof(char *) * totalPointers) ;
 
       for (i = nextPointer; i < totalPointers; i++)
 	PointersFreedOnExit [i] = NULL;
@@ -743,74 +623,48 @@ void addPointerFreedOnExit (char *pointerToFree)
   PointersFreedOnExit [nextPointer++] = pointerToFree ;
 }
 
-/* malloc a buffer and build the filename in it. */
-char *buildFilename (const char *directory, const char *fname)
-{
-  int len = 0 ;
-  char *p = NULL ;
-
-  if (fname == NULL)
-    return NULL ;
-
-  if (directory == NULL)
-    directory = "." ;
-  
-  len = strlen (directory) + strlen (fname) + 2 ;
-
-  if (len < pathMax(directory) - 2)
-    {
-      p = malloc (len + 1) ;
-      p [0] = '\0' ;
-      if (fname [0] != '/')
-        {
-          strcat (p,directory) ;
-          if (p [strlen(p) - 1] != '/')
-            strcat (p,"/") ;
-        }
-      strcat (p,fname) ;
-    }
-
-  return p ;
-}
-
-
 
 /* borrows heavily from the shrinkfile program by chongo. */
 bool shrinkfile (FILE *fp, long size, char *name, const char *mode)
 {
-  long currlen = ftell (fp) ;
+  long currlen = ftello (fp) ;
   char *tmpname ;
   char buffer [BUFSIZ] ;
   FILE *tmpFp ;
   int c ;
   int i ;
-
-  tmpname = malloc (pathMax(NULL) + 1) ;
-  sprintf (tmpname,"%s.XXXXXX",name) ;
-  mktemp (tmpname) ;
+  int fd ;
 
   if (currlen <= size)
     {
-      FREE(tmpname) ;
-      
       d_printf (1,"No need to shrink file (%s %ld vs %ld\n",
                name,size,currlen) ;
       return true ;
     }
 
-  /* create a temp file that will go away when closed. */
-  if ((tmpFp = fopen (tmpname,"w")) == NULL)
+  /* create a temp file. */
+  tmpname = concat (name,".XXXXXX",(char *)0) ;
+  fd = mkstemp (tmpname) ;
+
+  if (fd < 0)
     {
-      syslog (LOG_ERR,SHRINK_TEMP_OPEN,tmpname) ;
-      FREE (tmpname) ;
+      syswarn ("ME error creating temp shrink file for %s", name) ;
+      free (tmpname) ;
       return false ;
     }
 
-  if (fseek (fp,currlen - size,SEEK_SET) != 0)
+  if ((tmpFp = fdopen (fd,"w")) == NULL)
+    {
+      syswarn ("ME error opening temp shrink file %s", tmpname) ;
+      free (tmpname) ;
+      return false ;
+    }
+
+  if (fseeko (fp,currlen - size,SEEK_SET) != 0)
     {
       fclose (tmpFp) ;
-      syslog (LOG_ERR,SHRINK_SEEK,currlen - size,name) ;
-      FREE(tmpname) ;
+      warn ("ME error seeking to point %ld in %s", currlen - size, name) ;
+      free (tmpname) ;
       return false ;
     }
 
@@ -818,10 +672,10 @@ bool shrinkfile (FILE *fp, long size, char *name, const char *mode)
   while ((c = fgetc (fp)) != '\n')
     if (c == EOF)
       {
-        syslog (LOG_WARNING,SHRINK_NONL,name) ;
+        warn ("ME no newline in shrinking file %s", name) ;
         fclose (tmpFp) ;
-        fseek (fp,currlen,SEEK_SET) ;
-        FREE(tmpname) ;
+        fseeko (fp,currlen,SEEK_SET) ;
+        free (tmpname) ;
         return false ;
     }
 
@@ -831,46 +685,46 @@ bool shrinkfile (FILE *fp, long size, char *name, const char *mode)
       if (fwrite (buffer,1,i,tmpFp) != (size_t) i)
         {
           fclose (tmpFp) ;
-          syslog (LOG_ERR,SHRINK_WRITETMP,tmpname) ;
-          fseek (fp,currlen, SEEK_SET) ;
-          FREE(tmpname) ;
+          syswarn ("ME fwrite failed to temp shrink file %s", tmpname) ;
+          fseeko (fp,currlen, SEEK_SET) ;
+          free (tmpname) ;
           return false ;
         }
     }
 
   if (i < 0)
-    logAndExit (1,SHRINK_READ,name) ;
+    logAndExit (1,"ME fread failed on file %s: %s",name, strerror (errno)) ;
 
   fclose (tmpFp) ;
 
   if (unlink (name) != 0)
-    logAndExit (1,UNLINK_FAILED,name) ;
+    logAndExit (1,"ME oserr unlink %s: %s",name, strerror (errno)) ;
 
   /* we're in the same directory so this is ok. */
   if (rename (tmpname,name) != 0)
-    logAndExit (1,RENAME_FAILED,tmpname,name) ;
+    logAndExit (1,"ME oserr rename %s, %s: %s", tmpname, name,
+                strerror (errno)) ;
   
   if (freopen (name,mode,fp) != fp)
-    logAndExit (1,SHRINK_FREOPEN,name) ;
+    logAndExit (1,"ME freopen on shrink file failed %s: %s", name,
+                strerror (errno)) ;
 
-  fseek (fp,0,SEEK_END) ;
-  size = ftell (fp) ;
-  
-  syslog (LOG_WARNING,FILE_SHRUNK,name,currlen,size) ;
+  fseeko (fp,0,SEEK_END) ;
+  size = ftello (fp) ;
 
-  FREE(tmpname) ;
+  notice ("ME file %s shrunk from %ld to %ld", name, currlen, size) ;
+
+  free (tmpname) ;
   
   return true ;
 }
 
 
 
-long pathMax (const char *pathname)
+long pathMax (const char *pathname UNUSED)
 {
   static long rval = 0 ;
 
-  (void) pathname ;
-  
   if (rval > 0)
     return rval ;
   
