@@ -1,4 +1,4 @@
-/*  $Id: inndcomm.c 7597 2007-01-16 23:11:05Z eagle $
+/*  $Id: inndcomm.c 8307 2009-01-31 08:30:38Z iulius $
 **
 **  Library routines to let other programs control innd.
 */
@@ -23,9 +23,9 @@
 #endif
 
 #include "inn/innconf.h"
-#include "inndcomm.h"
-#include "libinn.h"
-#include "paths.h"
+#include "inn/inndcomm.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 
 static char			*ICCsockname = NULL;
 #ifdef HAVE_UNIX_DOMAIN_SOCKETS
@@ -54,7 +54,9 @@ int
 ICCopen(void)
 {
     int mask, oerrno, fd;
+#ifdef HAVE_UNIX_DOMAIN_SOCKETS
     int size = 65535;
+#endif
 
     if (innconf == NULL) {
 	if (!innconf_read(NULL)) {
@@ -70,7 +72,7 @@ ICCopen(void)
        at the same time, but the worst that will happen is that one process
        will delete the other's temporary socket). */
     if (ICCsockname == NULL)
-	ICCsockname = concatpath(innconf->pathrun, _PATH_TEMPSOCK);
+	ICCsockname = concatpath(innconf->pathrun, INN_PATH_TEMPSOCK);
     fd = mkstemp(ICCsockname);
     if (fd < 0) {
         ICCfailure = "mkstemp";
@@ -112,7 +114,7 @@ ICCopen(void)
     ICCserv.sun_family = AF_UNIX;
     strlcpy(ICCserv.sun_path, innconf->pathrun, sizeof(ICCserv.sun_path));
     strlcat(ICCserv.sun_path, "/", sizeof(ICCserv.sun_path));
-    strlcat(ICCserv.sun_path, _PATH_NEWSCONTROL, sizeof(ICCserv.sun_path));
+    strlcat(ICCserv.sun_path, INN_PATH_NEWSCONTROL, sizeof(ICCserv.sun_path));
 
 #else /* !HAVE_UNIX_DOMAIN_SOCKETS */
 
@@ -171,7 +173,7 @@ ICCserverpid(void)
     char		buff[SMBUF];
 
     pid = 1;
-    path = concatpath(innconf->pathrun, _PATH_SERVERPID);
+    path = concatpath(innconf->pathrun, INN_PATH_SERVERPID);
     F = fopen(path, "r");
     free(path);
     if (F != NULL) {
@@ -269,7 +271,7 @@ ICCcommand(char cmd, const char *argv[], char **replyp)
 	return -1;
     }
 #else /* !HAVE_UNIX_DOMAIN_SOCKETS */
-    path = concatpath(innconf->pathrun, _PATH_NEWSCONTROL);
+    path = concatpath(innconf->pathrun, INN_PATH_NEWSCONTROL);
     fd = open(path, O_WRONLY);
     free(path);
     if (fd < 0) {
