@@ -1,4 +1,4 @@
-/*  $Id: tdx-util.c 6289 2003-04-09 04:16:16Z rra $
+/*  $Id: tdx-util.c 8409 2009-04-11 22:13:41Z iulius $
 **
 **  Utility for managing a tradindexed overview spool.
 **
@@ -12,18 +12,19 @@
 #include "clibrary.h"
 #include <ctype.h>
 #include <dirent.h>
-#include <pwd.h>
 #include <sys/stat.h>
 
 #include "inn/buffer.h"
+#include "inn/libinn.h"
 #include "inn/history.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
+#include "inn/newsuser.h"
+#include "inn/ov.h"
+#include "inn/paths.h"
 #include "inn/vector.h"
-#include "libinn.h"
-#include "ov.h"
+#include "inn/wire.h"
 #include "ovinterface.h"
-#include "paths.h"
 #include "tdx-private.h"
 #include "tdx-structure.h"
 
@@ -88,16 +89,19 @@ dump_group_index(const char *group)
 **  Dump the overview data for a particular group.  If number is 0, dump the
 **  overview data for all current articles; otherwise, only dump the data for
 **  that particular article.  Include the article number, token, arrived time,
-**  and expires time (if any) in the overview data as additional fields.
+**  and expires time (if any) in the overview data as additional fields
+**  normally.  If overchan is true, instead write out the data in the format
+**  that overchan would expect as input.
 */
 static void
-dump_overview(const char *group, ARTNUM number)
+dump_overview(const char *group, ARTNUM low, ARTNUM high, bool overchan)
 {
     struct group_index *index;
     struct group_data *data;
     struct group_entry *entry;
     struct article article;
     struct search *search;
+    char *p;
     char datestring[256];
 
     index = tdx_index_open(OV_READ);
@@ -115,27 +119,40 @@ dump_overview(const char *group, ARTNUM number)
     }
     data->refcount++;
 
-    if (number != 0)
-        search = tdx_search_open(data, number, number, entry->high);
-    else
-        search = tdx_search_open(data, entry->low, entry->high, entry->high);
+    if (low == 0)
+        low = entry->low;
+    if (high == 0)
+        high = entry->high;
+    search = tdx_search_open(data, low, high, entry->high);
+
     if (search == NULL) {
-        if (number != 0)
+        if (low == high)
             puts("Article not found");
         else
-            warn("cannot open search in %s: %lu - %lu", group, entry->low,
-                 entry->high);
+            warn("cannot open search in %s: %lu - %lu", group, low, high);
         return;
     }
     while (tdx_search(search, &article)) {
-        fwrite(article.overview, article.overlen - 2, 1, stdout);
-        printf("\tArticle: %lu\tToken: %s", article.number,
-               TokenToText(article.token));
-        makedate(article.arrived, true, datestring, sizeof(datestring));
-        printf("\tArrived: %s", datestring);
-        if (article.expires != 0) {
-            makedate(article.expires, true, datestring, sizeof(datestring));
-            printf("\tExpires: %s", datestring);
+        if (overchan) {
+            p = memchr(article.overview, '\t', article.overlen - 3);
+            if (p == NULL)
+                continue;
+            p++;
+            printf("%s %lu %lu ", TokenToText(article.token),
+                   (unsigned long) article.arrived,
+                   (unsigned long) article.expires);
+            fwrite(p, article.overlen - 2 - (p - article.overview), 1, stdout);
+        } else {
+            fwrite(article.overview, article.overlen - 2, 1, stdout);
+            printf("\tArticle: %lu\tToken: %s", article.number,
+                   TokenToText(article.token));
+            makedate(article.arrived, true, datestring, sizeof(datestring));
+            printf("\tArrived: %s", datestring);
+            if (article.expires != 0) {
+                makedate(article.expires, true, datestring,
+                         sizeof(datestring));
+                printf("\tExpires: %s", datestring);
+            }
         }
         printf("\n");
     }
@@ -232,6 +249,26 @@ article_list(const char *directory)
 }
 
 
+/*
+**  Create a new newsgroup in the overview database.  Takes the group flag and
+**  the high and low water marks.
+*/
+static void
+group_create(const char *group, ARTNUM low, ARTNUM high, char flag)
+{
+    struct group_index *index;
+
+    index = tdx_index_open(true);
+    if (index == NULL)
+        die("cannot open group index");
+    if (low == 0)
+        low = 1;
+    if (!tdx_index_add(index, group, low, high, &flag))
+        die("cannot create group %s", group);
+    tdx_index_close(index);
+}
+
+
 /*
 **  Rebuild the overview data for a particular group.  Takes a path to a
 **  directory containing all the articles, as individual files, that should be
@@ -242,8 +279,8 @@ static void
 group_rebuild(const char *group, const char *path)
 {
     char *filename, *histpath, *article, *wireformat, *p;
-    size_t size, file;
-    int flags, length;
+    size_t size, length, file;
+    int flags;
     struct buffer *overview = NULL;
     struct vector *extra, *files;
     struct history *history;
@@ -271,14 +308,14 @@ group_rebuild(const char *group, const char *path)
     if (!tdx_index_rebuild_start(index, entry))
         die("cannot start index rebuild for %s", group);
 
-    histpath = concatpath(innconf->pathdb, _PATH_HISTORY);
+    histpath = concatpath(innconf->pathdb, INN_PATH_HISTORY);
     flags = HIS_RDONLY | HIS_ONDISK;
     history = HISopen(histpath, innconf->hismethod, flags);
     if (history == NULL)
         sysdie("cannot open history %s", histpath);
     free(histpath);
 
-    extra = overview_extra_fields();
+    extra = overview_extra_fields(true);
     files = article_list(path);
 
     info.count = 0;
@@ -298,7 +335,7 @@ group_rebuild(const char *group, const char *path)
            convert it.  We only check the first line ending. */
         p = strchr(article, '\n');
         if (p != NULL && (p == article || p[-1] != '\r')) {
-            wireformat = ToWireFmt(article, size, (size_t *)&length);
+            wireformat = wire_from_native(article, size, &length);
             free(article);
             article = wireformat;
             size = length;
@@ -347,21 +384,43 @@ group_rebuild(const char *group, const char *path)
 
 
 /*
-**  Change to the news user if possible, and if not, die.  Used for operations
-**  that may change the overview files so as not to mess up the ownership.
+**  Parse an article number or range, returning the low and high values in the
+**  provided arguments.  Returns true if the number or range was parsed
+**  successfully, false otherwise.  Allows such constructs as "-", "20-", or
+**  "-50" and leaves the unspecified ends of the range set to zero.
+**
+**  This code is similar to code in nnrpd and possibly should move into libinn
+**  as common code.
 */
-static void
-setuid_news(void)
+static bool
+parse_range(char *range, ARTNUM *low, ARTNUM *high)
 {
-    struct passwd *pwd;
+    char *hyphen, *end;
 
-    pwd = getpwnam(NEWSUSER);
-    if (pwd == NULL)
-        die("can't resolve %s to a UID (account doesn't exist?)", NEWSUSER);
-    if (getuid() == 0)
-        setuid(pwd->pw_uid);
-    if (getuid() != pwd->pw_uid)
-        die("must be run as %s", NEWSUSER);
+    *low = 0;
+    *high = 0;
+    hyphen = strchr(range, '-');
+    if (hyphen == NULL) {
+        *low = strtoul(range, &end, 10);
+        if (*low == 0 || *end != '\0')
+            return false;
+        *high = *low;
+        return true;
+    } else {
+        *hyphen = '\0';
+        if (*range != '\0') {
+            *low = strtoul(range, &end, 10);
+            if (*low == 0 || *end != '\0')
+                return false;
+        }
+        if (hyphen[1] != '\0') {
+            *high = strtoul(hyphen + 1, &end, 10);
+            if (*high == 0 || *end != '\0')
+                return false;
+        }
+        *hyphen = '-';
+        return true;
+    }
 }
 
 
@@ -373,10 +432,12 @@ int
 main(int argc, char *argv[])
 {
     int option;
+    char flag = 'y';
     char mode = '\0';
     const char *newsgroup = NULL;
     const char *path = NULL;
-    ARTNUM article = 0;
+    ARTNUM artlow = 0;
+    ARTNUM arthigh = 0;
 
     message_program_name = "tdx-util";
 
@@ -385,12 +446,14 @@ main(int argc, char *argv[])
 
     /* Parse options. */
     opterr = 0;
-    while ((option = getopt(argc, argv, "a:n:p:AFR:gio")) != EOF) {
+    while ((option = getopt(argc, argv, "a:f:n:p:AFR:cgiOo")) != EOF) {
         switch (option) {
         case 'a':
-            article = strtoul(optarg, NULL, 10);
-            if (article == 0)
-                die("invalid article number %s", optarg);
+            if (!parse_range(optarg, &artlow, &arthigh))
+                die("invalid article number or range %s", optarg);
+            break;
+        case 'f':
+            flag = optarg[0];
             break;
         case 'n':
             newsgroup = optarg;
@@ -414,6 +477,11 @@ main(int argc, char *argv[])
             mode = 'R';
             path = optarg;
             break;
+        case 'c':
+            if (mode != '\0')
+                die("only one mode option allowed");
+            mode = 'c';
+            break;
         case 'g':
             if (mode != '\0')
                 die("only one mode option allowed");
@@ -424,6 +492,11 @@ main(int argc, char *argv[])
                 die("only one mode option allowed");
             mode = 'i';
             break;
+        case 'O':
+            if (mode != '\0')
+                die("only one mode option allowed");
+            mode = 'O';
+            break;
         case 'o':
             if (mode != '\0')
                 die("only one mode option allowed");
@@ -435,8 +508,8 @@ main(int argc, char *argv[])
         }
     }
 
-    /* Modes g and o require a group be specified. */
-    if ((mode == 'g' || mode == 'o' || mode == 'R') && newsgroup == NULL)
+    /* Some modes require a group be specified. */
+    if (strchr("cgoOR", mode) != NULL && newsgroup == NULL)
         die("group must be specified for -%c", mode);
 
     /* Run the specified function. */
@@ -445,21 +518,31 @@ main(int argc, char *argv[])
         tdx_index_audit(false);
         break;
     case 'F':
-        setuid_news();
+        if (getenv("INN_TESTSUITE") == NULL)
+            ensure_news_user_grp(true, true);
         tdx_index_audit(true);
         break;
     case 'R':
-        setuid_news();
+        if (getenv("INN_TESTSUITE") == NULL)
+            ensure_news_user_grp(true, true);
         group_rebuild(newsgroup, path);
         break;
+    case 'c':
+        if (getenv("INN_TESTSUITE") == NULL)
+            ensure_news_user_grp(true, true);
+        group_create(newsgroup, artlow, arthigh, flag);
+        break;
     case 'i':
         dump_index(newsgroup);
         break;
     case 'g':
         dump_group_index(newsgroup);
         break;
+    case 'O':
+        dump_overview(newsgroup, artlow, arthigh, true);
+        break;
     case 'o':
-        dump_overview(newsgroup, article);
+        dump_overview(newsgroup, artlow, arthigh, false);
         break;
     default:
         die("a mode option must be specified");
